package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.AllocationExpression;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.JavadocQualifiedTypeReference;
import org.eclipse.jdt.internal.compiler.ast.Statement;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ProblemMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class JavadocAllocationExpression extends AllocationExpression {
  public int tagSourceStart;
  
  public int tagSourceEnd;
  
  public int tagValue;
  
  public int memberStart;
  
  public char[][] qualification;
  
  public JavadocAllocationExpression(int start, int end) {
    this.sourceStart = start;
    this.sourceEnd = end;
    this.bits |= 0x8000;
  }
  
  public JavadocAllocationExpression(long pos) {
    this((int)(pos >>> 32L), (int)pos);
  }
  
  TypeBinding internalResolveType(Scope scope) {
    this.constant = Constant.NotAConstant;
    if (this.type == null) {
      this.resolvedType = (TypeBinding)scope.enclosingSourceType();
    } else if (scope.kind == 3) {
      this.resolvedType = this.type.resolveType((ClassScope)scope);
    } else {
      this.resolvedType = this.type.resolveType((BlockScope)scope, true);
    } 
    this.argumentTypes = Binding.NO_PARAMETERS;
    boolean hasTypeVarArgs = false;
    if (this.arguments != null) {
      this.argumentsHaveErrors = false;
      int length = this.arguments.length;
      this.argumentTypes = new TypeBinding[length];
      for (int i = 0; i < length; i++) {
        Expression argument = this.arguments[i];
        if (scope.kind == 3) {
          this.argumentTypes[i] = argument.resolveType((ClassScope)scope);
        } else {
          this.argumentTypes[i] = argument.resolveType((BlockScope)scope);
        } 
        if (this.argumentTypes[i] == null) {
          this.argumentsHaveErrors = true;
        } else if (!hasTypeVarArgs) {
          hasTypeVarArgs = this.argumentTypes[i].isTypeVariable();
        } 
      } 
      if (this.argumentsHaveErrors)
        return null; 
    } 
    if (this.resolvedType == null)
      return null; 
    this.resolvedType = scope.environment().convertToRawType(this.type.resolvedType, true);
    SourceTypeBinding enclosingType = scope.enclosingSourceType();
    if (enclosingType != null && enclosingType.isCompatibleWith(this.resolvedType))
      this.bits |= 0x4000; 
    ReferenceBinding allocationType = (ReferenceBinding)this.resolvedType;
    this.binding = scope.getConstructor(allocationType, this.argumentTypes, (InvocationSite)this);
    if (!this.binding.isValidBinding()) {
      ReferenceBinding enclosingTypeBinding = allocationType;
      MethodBinding contructorBinding = this.binding;
      while (!contructorBinding.isValidBinding() && (enclosingTypeBinding.isMemberType() || enclosingTypeBinding.isLocalType())) {
        enclosingTypeBinding = enclosingTypeBinding.enclosingType();
        contructorBinding = scope.getConstructor(enclosingTypeBinding, this.argumentTypes, (InvocationSite)this);
      } 
      if (contructorBinding.isValidBinding())
        this.binding = contructorBinding; 
    } 
    if (!this.binding.isValidBinding()) {
      MethodBinding methodBinding = scope.getMethod(this.resolvedType, this.resolvedType.sourceName(), this.argumentTypes, (InvocationSite)this);
      if (methodBinding.isValidBinding()) {
        this.binding = methodBinding;
      } else {
        if (this.binding.declaringClass == null)
          this.binding.declaringClass = allocationType; 
        scope.problemReporter().javadocInvalidConstructor((Statement)this, this.binding, scope.getDeclarationModifiers());
      } 
      return this.resolvedType;
    } 
    if (this.binding.isVarargs()) {
      int length = this.argumentTypes.length;
      if (this.binding.parameters.length != length || !this.argumentTypes[length - 1].isArrayType()) {
        ProblemMethodBinding problemMethodBinding = new ProblemMethodBinding(this.binding, this.binding.selector, this.argumentTypes, 1);
        scope.problemReporter().javadocInvalidConstructor((Statement)this, (MethodBinding)problemMethodBinding, scope.getDeclarationModifiers());
      } 
    } else if (hasTypeVarArgs) {
      ProblemMethodBinding problemMethodBinding = new ProblemMethodBinding(this.binding, this.binding.selector, this.argumentTypes, 1);
      scope.problemReporter().javadocInvalidConstructor((Statement)this, (MethodBinding)problemMethodBinding, scope.getDeclarationModifiers());
    } else if (this.binding instanceof ParameterizedMethodBinding) {
      ParameterizedMethodBinding paramMethodBinding = (ParameterizedMethodBinding)this.binding;
      if (paramMethodBinding.hasSubstitutedParameters()) {
        int length = this.argumentTypes.length;
        for (int i = 0; i < length; i++) {
          if (TypeBinding.notEquals(paramMethodBinding.parameters[i], this.argumentTypes[i]) && 
            TypeBinding.notEquals(paramMethodBinding.parameters[i].erasure(), this.argumentTypes[i].erasure())) {
            ProblemMethodBinding problemMethodBinding = new ProblemMethodBinding(this.binding, this.binding.selector, this.argumentTypes, 1);
            scope.problemReporter().javadocInvalidConstructor((Statement)this, (MethodBinding)problemMethodBinding, scope.getDeclarationModifiers());
            break;
          } 
        } 
      } 
    } else if (this.resolvedType.isMemberType()) {
      int length = this.qualification.length;
      if (length > 1) {
        ReferenceBinding enclosingTypeBinding = allocationType;
        if (this.type instanceof JavadocQualifiedTypeReference && ((JavadocQualifiedTypeReference)this.type).tokens.length != length) {
          scope.problemReporter().javadocInvalidMemberTypeQualification(this.memberStart + 1, this.sourceEnd, scope.getDeclarationModifiers());
        } else {
          int idx = length;
          do {
          
          } while (idx > 0 && CharOperation.equals(this.qualification[--idx], enclosingTypeBinding.sourceName) && (enclosingTypeBinding = enclosingTypeBinding.enclosingType()) != null);
          if (idx > 0 || enclosingTypeBinding != null)
            scope.problemReporter().javadocInvalidMemberTypeQualification(this.memberStart + 1, this.sourceEnd, scope.getDeclarationModifiers()); 
        } 
      } 
    } 
    if (isMethodUseDeprecated(this.binding, scope, true))
      scope.problemReporter().javadocDeprecatedMethod(this.binding, (ASTNode)this, scope.getDeclarationModifiers()); 
    return (TypeBinding)allocationType;
  }
  
  public boolean isSuperAccess() {
    return ((this.bits & 0x4000) != 0);
  }
  
  public TypeBinding resolveType(BlockScope scope) {
    return internalResolveType((Scope)scope);
  }
  
  public TypeBinding resolveType(ClassScope scope) {
    return internalResolveType((Scope)scope);
  }
  
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    if (visitor.visit(this, scope)) {
      if (this.typeArguments != null)
        for (int i = 0, typeArgumentsLength = this.typeArguments.length; i < typeArgumentsLength; i++)
          this.typeArguments[i].traverse(visitor, scope);  
      if (this.type != null)
        this.type.traverse(visitor, scope); 
      if (this.arguments != null)
        for (int i = 0, argumentsLength = this.arguments.length; i < argumentsLength; i++)
          this.arguments[i].traverse(visitor, scope);  
    } 
    visitor.endVisit(this, scope);
  }
  
  public void traverse(ASTVisitor visitor, ClassScope scope) {
    if (visitor.visit(this, scope)) {
      if (this.typeArguments != null)
        for (int i = 0, typeArgumentsLength = this.typeArguments.length; i < typeArgumentsLength; i++)
          this.typeArguments[i].traverse(visitor, scope);  
      if (this.type != null)
        this.type.traverse(visitor, scope); 
      if (this.arguments != null)
        for (int i = 0, argumentsLength = this.arguments.length; i < argumentsLength; i++)
          this.arguments[i].traverse(visitor, scope);  
    } 
    visitor.endVisit(this, scope);
  }
}
